@@ -1,6 +1,6 @@
 /* command.c
  *
- * $Id: command.c,v 1.26 1999/09/07 20:07:53 nisse Exp $ */
+ * $Id: command.c,v 1.27 1999/09/21 22:25:03 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -353,7 +353,7 @@ do_once_continuation(struct command_continuation *s,
 }
 
 struct command_continuation *
-make_once_continution(const char *msg,
+make_once_continuation(const char *msg,
 		      struct command_continuation *up)
 {
   NEW(once_continuation, self);
