@@ -1,6 +1,6 @@
 /* server_keyexchange.c
  *
- * $Id: server_keyexchange.c,v 1.31 1999/09/21 23:02:31 nisse Exp $ */
+ * $Id: server_keyexchange.c,v 1.32 1999/10/02 20:15:12 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -117,7 +117,7 @@ do_handle_dh_init(struct packet_handler *c,
 #if DATAFELLOWS_WORKAROUNDS
   if (! (connection->peer_flags & PEER_SEND_NO_DEBUG))
 #endif
-    send_verbose(connection, "Key exchange successful!", 0);
+    send_verbose(connection->write, "Key exchange successful!", 0);
 
   if (connection->established)
     {
