@@ -1,6 +1,6 @@
 /* tcpforward.c
  *
- * $Id: tcpforward.c,v 1.30 1999/09/20 22:35:12 nisse Exp $
+ * $Id: tcpforward.c,v 1.31 1999/10/02 20:40:52 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -179,6 +179,9 @@ struct ssh_channel *make_tcpip_channel(struct io_fd *socket, UINT32 max_window)
   return &self->super;
 }
 
+/* FIXME: Where should we install a proper I/O-error handler, that
+ * closes the channel? */
+
 void tcpip_channel_start_io(struct ssh_channel *c)
 {
   CAST(tcpip_channel, channel, c);
@@ -192,7 +195,7 @@ void tcpip_channel_start_io(struct ssh_channel *c)
 		make_channel_read_data(&channel->super),
 		/* FIXME: Make this configurable */
 		SSH_MAX_PACKET * 10, /* self->block_size, */
-		make_channel_close_callback(&channel->super));
+		make_channel_read_close_callback(&channel->super));
 
   /* Start receiving */
   channel_start_receive(&channel->super);
